package Tests::Base;

use Sentinel::Base;
@ISA = qw( Sentinel::Base );

use IO::Socket::INET;

%AUTOFIELDS = (
	message => undef,
	host => undef,
	args => undef,
	cdata => undef,

	probe_code => undef,
	probe_message => undef,
	probe_port => 9122,
	probe_version => 1.3, # MUST MATCH PROBE VERSION ON CLIENTS, PROTOCOL VERSION CHECK
	lib_host => '0.0.0.0',
	lib_port => 9123
);

sub execute
{
	my $self = shift;
	$self -> logmsg( "Plugin '" . ref( $self ) . "' did not provide an execute method" );
	$self -> logmsg( "  args --> ('" .
		join( "','", @_ ) .
		"')" );
	$self -> message( "Plugin did not provide an execute method" );
	return( 1 );
}

sub host_addr
{
	return( shift() -> host() -> addr() );
}

sub probe
{
	# Takes the probe name, returns code/message
	# If no probe name passed, default is the same as the plugin name
	my $self = shift;
	my $probe = shift();
	my( $s, $line );

	if( ! $probe )
	{
		$probe = ref( $self );
		$probe =~ s/^.*://;
	}
	$self -> logmsg( "Probe is: $probe" );

	# Stupid IO::Socket::INET tried to throw a croak
	eval{
		local $SIG{ __DIE__ } = sub { 0 };
		$s = IO::Socket::INET -> new(
			PeerAddr => $self -> host_addr(),
			PeerPort => $self -> probe_port(),
			Proto => 'tcp' );
	};

	if( ! $s )
	{
		$self -> logmsg( "Couldn't create socket: $!" );
		$self -> probe_code( 501 ),
		$self -> probe_message( "Couldn't create socket: $!" );
		return( $self -> probe_code() );
	}

	# OK, do conversation
	$s -> print( $self -> probe_version() . "\r\n" );
	$line = $s -> getline(); 
	chomp( $line );
	$line =~ s/\r$//;
	( $code, $message ) = $line =~ /^(\d+)\s+(\S.+)/;
	if( $code != 200 )
	{
		# Error
		$self -> logmsg( "Error: $code/$message" );
		$self -> probe_code( $code );
		$self -> probe_message( $message );
		return( $self -> probe_code() );
	}

	# Instruct what and where to download
	$s -> print( "$probe " . $self -> lib_host() . ":" . $self -> lib_port() . "\r\n" );

	# Status of loading probe
	$line = $s -> getline(); 
	chomp( $line );
	$line =~ s/\r$//;
	( $code, $message ) = $line =~ /^(\d+)\s+(\S.+)/;
	if( $code != 200 )
	{
		# Error
		$self -> logmsg( "Error: $code/$message" );
		$self -> probe_code( $code );
		$self -> probe_message( $message );
		return( $self -> probe_code() );
	}

	# Pass arguments
	$s -> print( join( "\t", @_ ) ) if @_;
	$s -> print( "\r\n" );

	# Returns final message code (this is the real probe)
	$line = $s -> getline(); 
	chomp( $line );
	$line =~ s/\r$//;
	( $code, $message ) = $line =~ /^(\d+)\s+(\S.+)/;

	# Final message code
	$self -> logmsg( "Final return of probe: $code/$message" );
	$self -> probe_code( $code );
	$self -> probe_message( $message );
	return( $self -> probe_code() );
}

sub probe_return
{
	# Many probes are the whole test, this will return 0 if the
	# probe returns 200, or set the message and return 1 if
	# the probe reports failure
	my $self = shift;

	$self -> message( $self -> probe_message() );
	if( $self -> probe( @_ ) == 200 )
	{
		return( 0 );
	}else
	{
		# Failed
		return( 1 );
	}
}

sub opt
{
	return( shift() -> args() -> { shift() } );
}

1;

__END__

=head1 NAME

Tests::Base - Base inherited class for sentinel test plugins

=head1 SYNOPSIS

	package Tests::mytest;

	use Tests::Base;
	@ISA = qw( Tests::Base );

	sub execute
	{
		my $self = shift;

		.. do something ..

		return( 0 );
	}

=head1 DESCRIPTION

This is the base class for sentinel plugins, it provides several methods for
inherited classes, which are essentially plugins.  All plugins should declare
and execute method.  It will be passed no arguments, but arguments can be accessed
through methods.

=head2 opt

Gets the value of a set options from the config file.  For example, if the config
contained:

	<mytest arg="value"/>

Then opt( 'arg' ) would be the string "value".

=head2 probe

Executes a probe on the current host this test is running for.  This will initiate a
connection to the test host, request for it to download the probe, and the return
message and code will be set.  The plugin can get the return code through the probe_code
and probe_message methods.  Most probes will use code 200 for success, but YMMV.

=head2 host_addr

The address of the test host, for example if you need to ping it.

=head2 message

This should be set if the plugin will return a failure code.  The message will be relayed
back to any notify sources.  i.e. if the plugin is pinging a host and gets no response, it
should set the message to something like this:

	$self -> message( "Host not responding to PING" );


Then return 1.

=head2 probe_return

Many plugins consist of only the return of the probe.  This can be used for those cases,
if the probe returns 200 then this will return 0, if not then this will set the message
to the message returned by the probe and return 1.  If wrapped in a return, this is all
that's needed in an execute block:

	sub execute
	{
		my $self = shift;
		# Returns 1 if probe returns != 200 and auto-sets message
		return( $self -> probe_return( undef, $self -> opt( 'somearg' ) ) );
	}

=head2 cdata

Returns the cdata for this test block.  Note that the data may contain a lot of
whitespace, it will be raw and may require cleansing.

=head1 EXAMPLE

This is an example probe.  It will take a single argument 'port' and make sure that the port
is open on the test host.

	package Tests::ping_port;

	use Tests::Base;
	@ISA = qw( Tests::Base );

	$VERSION = 1.00;

	use IO::Socket::INET;

	sub execute
	{
		my $self = shift;
		$port = $self -> opt( 'port' ) || 'echo';

		my $s;
		eval{
			local $SIG{ __DIE__ } = sub { 0 };
			$s = IO::Socket::INET -> new(
			PeerAddr => $self -> host_addr(),
			PeerPort => $port;
			Proto => 'tcp' );
		};

		if( $s )
		{
			return( 0 );
		}else
		{
			$self -> message( "Error connecting to port '$port': $!" );
			return( 1 );
		}
	}

	1;

To use this plugin, it would need to be saved in Tests/ping_port.pm, and the
sentinel.conf would need to have:

	<host addr="some.host.com" name="Test host">
		<ping_port port="80"/>
	</host>

This would instruct the sentinel to load the ping_port plugin, and the plugin
would try and connect to port 80 on some.host.com.  If the connection couldn't
be made, it would set the message to an error and return an error code, causing
the sentinel to send out notify messages.

=head1 SEE ALSO

sentinel, probe, librarian, sentinel.conf

=head1 AUTHOR

Steve Slaven - http://hoopajoo.net
